/*=======================================================================

 User-supplied f (ODEs) routine. Compute the function ydot = f(t,y)

-----------------------------------------------------------------------*/
/* Header files with descriptions of the contents used */

#include <math.h>                    /* Standard math functions */
#include <cvode/cvode.h>             /* CVODE functions and constants */
#include <cvode/cvode_dense.h>       /* Prototype for CVDense solver */
#include <nvector/nvector_serial.h>  /* Serial N_Vector types, functions, macros */
#include <sundials/sundials_dense.h> /* Definition of type DlsMat (dense matrix) */
#include <sundials/sundials_types.h> /* Definition of type realtype */

/*-----------------------------------------------------------------------*/

/* Type definition for user-supplied data passed to the solver functions */

typedef struct {
  realtype *rate, n_H, T_g, x_e;
} *User_Data;

/*-----------------------------------------------------------------------*/


int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
{
  realtype *x, *ode, *rate;
  realtype n_H, x_e, loss, form, zeta_H, zeta_H2, zeta_He;
  User_Data data;

  /* Obtain pointers to the y and ydot vector data arrays */
  x = NV_DATA_S(y);
  ode = NV_DATA_S(ydot);

  /* Retrieve the array of reaction rate coefficients and
   * the total number density from the user-supplied data */
  data = (User_Data) user_data;
  rate = data->rate;
  n_H = data->n_H;

  /* The electron abundance is a conserved quantity, given by the sum
   * of the abundances of all ionized species in the chemical network */
  x_e = x[0]-x[1]+x[2]+2*x[3]+x[7]+x[9]+x[10]+x[11]+x[13]+x[14]+x[15]+x[16]+x[17]+x[19]+x[22]+x[23]+x[24]+x[25]+x[26]+x[27]+x[28]+x[29]+x[30]+x[31]+x[32]+x[33]+x[35]+x[37]+x[39]+x[40]+x[41]+x[43]+x[45]+x[48];

  /* The X-ray secondary ionization rates depend on the mean energies
   * required to ionize H or H2 in a neutral gas mixture, 1/(W_i*x_i) */
  zeta_H  = 1.0/(39.8*(1.0+12.2*pow(x_e,0.866))*(x[56]+1.89*x[55]));
  zeta_H2 = 1.0/(41.9*(1.0+6.72*pow((1.83*x_e/(1.0+0.83*x_e)),0.824))*(x[55]+0.53*x[56]));
  zeta_He = 1.0/(487.*(1.0+12.5*pow(x_e,0.994))*(x[50]));

  /* The ODEs created by MakeRates begin here... */
  loss = -rate[183]*x_e*n_H;
  form = +rate[571]*x[17]*x[55]*n_H;
  ode[0] = form+x[0]*loss;
  loss = -rate[0]*x[49]*n_H-rate[2]*x[56]*n_H-rate[3]*x[54]*n_H-rate[140]-rate[413]*x[35]*n_H-rate[414]*x[39]*n_H-rate[415]*x[40]*n_H-rate[416]*x[37]*n_H-rate[417]*x[2]*n_H-rate[418]*x[33]*n_H-rate[419]*x[41]*n_H-rate[420]*x[9]*n_H-rate[421]*x[31]*n_H-rate[422]*x[24]*n_H-rate[423]*x[48]*n_H-rate[424]*x[23]*n_H-rate[553];
  form = +rate[578]*x[53]*x_e*n_H;
  ode[1] = form+x[1]*loss;
  loss = -rate[171]*x_e*n_H-rate[172]*x_e*n_H-rate[313]*x[49]*n_H-rate[314]*x[38]*n_H-rate[315]*x[44]*n_H-rate[316]*x[36]*n_H-rate[317]*x[46]*n_H-rate[318]*x[51]*n_H-rate[319]*x[34]*n_H-rate[320]*x[8]*n_H-rate[321]*x[42]*n_H-rate[322]*x[18]*n_H-rate[323]*x[4]*n_H-rate[324]*x[5]*n_H-rate[325]*x[47]*n_H-rate[326]*x[6]*n_H-rate[327]*x[54]*n_H-rate[328]*x[54]*n_H-rate[329]*x[52]*n_H-rate[330]*x[12]*n_H-rate[331]*x[53]*n_H-rate[332]*x[20]*n_H-rate[417]*x[1]*n_H-rate[537]-rate[538];
  form = +rate[281]*x[14]*x[55]*n_H;
  ode[2] = form+x[2]*loss;
  loss = -rate[632]*x_e*n_H-rate[633]*x[56]*n_H;
  form = +rate[590]*x[53]+rate[591]*x[48];
  ode[3] = form+x[3]*loss;
  loss = -rate[131]-rate[273]*x[40]*n_H-rate[323]*x[2]*n_H-rate[373]*x[9]*n_H-rate[374]*x[9]*n_H-rate[478]*x[56]*n_H-rate[492]*x[54]*n_H-rate[540]-rate[621]*x[55]*zeta_H2;
  form = +rate[163]*x[11]*x_e*n_H+rate[443]*x[38]*x[53]*n_H;
  ode[4] = form+x[4]*loss;
  loss = -rate[49]*x[40]*n_H-rate[132]-rate[324]*x[2]*n_H-rate[335]*x[33]*n_H-rate[348]*x[41]*n_H-rate[375]*x[9]*n_H-rate[376]*x[9]*n_H-rate[543]-rate[544]-rate[622]*x[55]*zeta_H2;
  form = +rate[169]*x[10]*x_e*n_H+rate[183]*x[0]*x_e*n_H;
  ode[5] = form+x[5]*loss;
  loss = -rate[18]*x[35]*n_H-rate[22]*x[43]*n_H-rate[29]*x[39]*n_H-rate[51]*x[40]*n_H-rate[68]*x[29]*n_H-rate[82]*x[25]*n_H-rate[83]*x[37]*n_H-rate[84]*x[28]*n_H-rate[85]*x[27]*n_H-rate[86]*x[48]*n_H-rate[87]*x[23]*n_H-rate[135]-rate[256]*x[16]*n_H-rate[326]*x[2]*n_H-rate[546]-rate[627]*x[55]*zeta_H2;
  form = +rate[421]*x[1]*x[31]*n_H+rate[585]*x[31]*x_e*n_H;
  ode[6] = form+x[6]*loss;
  loss = -rate[190]*x_e*n_H;
  form = +rate[305]*x[29]*x[53]*n_H+rate[332]*x[2]*x[20]*n_H+rate[352]*x[20]*x[41]*n_H;
  ode[7] = form+x[7]*loss;
  loss = -rate[46]*x[40]*n_H-rate[126]-rate[195]*x[35]*n_H-rate[320]*x[2]*n_H-rate[346]*x[41]*n_H-rate[366]*x[9]*n_H-rate[367]*x[9]*n_H-rate[368]*x[9]*n_H-rate[530]-rate[623]*x[55]*zeta_H2;
  form = +rate[174]*x[15]*x_e*n_H+rate[450]*x[44]*x[53]*n_H+rate[582]*x[11]*x_e*n_H;
  ode[8] = form+x[8]*loss;
  loss = -rate[66]*x[55]*n_H-rate[74]*x[56]*n_H-rate[76]*x[49]*n_H-rate[77]*x[36]*n_H-rate[78]*x[46]*n_H-rate[79]*x[42]*n_H-rate[80]*x[18]*n_H-rate[81]*x[21]*n_H-rate[296]*x[55]*n_H-rate[355]*x[38]*n_H-rate[356]*x[38]*n_H-rate[357]*x[44]*n_H-rate[358]*x[36]*n_H-rate[359]*x[36]*n_H-rate[360]*x[36]*n_H-rate[361]*x[36]*n_H-rate[362]*x[46]*n_H-rate[363]*x[51]*n_H-rate[364]*x[34]*n_H-rate[365]*x[34]*n_H-rate[366]*x[8]*n_H-rate[367]*x[8]*n_H-rate[368]*x[8]*n_H-rate[369]*x[42]*n_H-rate[370]*x[42]*n_H-rate[371]*x[18]*n_H-rate[372]*x[18]*n_H-rate[373]*x[4]*n_H-rate[374]*x[4]*n_H-rate[375]*x[5]*n_H-rate[376]*x[5]*n_H-rate[377]*x[47]*n_H-rate[378]*x[21]*n_H-rate[379]*x[52]*n_H-rate[380]*x[12]*n_H-rate[381]*x[20]*n_H-rate[382]*x[20]*n_H-rate[420]*x[1]*n_H-rate[584]*x_e*n_H;
  form = +rate[112]*x[50]+rate[134]*x[50]+rate[594]*x[50]*zeta_He;
  ode[9] = form+x[9]*loss;
  loss = -rate[169]*x_e*n_H-rate[170]*x_e*n_H;
  form = +rate[324]*x[2]*x[5]*n_H+rate[335]*x[5]*x[33]*n_H+rate[348]*x[5]*x[41]*n_H+rate[412]*x[26]*x[53]*n_H;
  ode[10] = form+x[10]*loss;
  loss = -rate[162]*x_e*n_H-rate[163]*x_e*n_H-rate[582]*x_e*n_H;
  form = +rate[46]*x[8]*x[40]*n_H+rate[237]*x[39]*x[47]*n_H+rate[240]*x[44]*x[48]*n_H+rate[323]*x[2]*x[4]*n_H;
  ode[11] = form+x[11]*loss;
  loss = -rate[55]*x[40]*n_H-rate[141]-rate[330]*x[2]*n_H-rate[336]*x[33]*n_H-rate[350]*x[41]*n_H-rate[380]*x[9]*n_H-rate[431]*x[49]*n_H-rate[482]*x[56]*n_H-rate[496]*x[54]*n_H-rate[554]-rate[555]-rate[615]*x[55]*zeta_H2;
  form = +rate[189]*x[17]*x_e*n_H+rate[311]*x[17]*x[18]*n_H+rate[502]*x[47]*x[53]*n_H+rate[503]*x[20]*x[53]*n_H;
  ode[12] = form+x[12]*loss;
  loss = -rate[193]*x_e*n_H;
  form = +rate[55]*x[12]*x[40]*n_H+rate[312]*x[18]*x[23]*n_H+rate[376]*x[5]*x[9]*n_H+rate[411]*x[18]*x[48]*n_H+rate[554]*x[12];
  ode[13] = form+x[13]*loss;
  loss = -rate[58]*x[38]*n_H-rate[59]*x[36]*n_H-rate[60]*x[46]*n_H-rate[61]*x[51]*n_H-rate[62]*x[42]*n_H-rate[63]*x[18]*n_H-rate[64]*x[21]*n_H-rate[65]*x[52]*n_H-rate[73]*x[56]*n_H-rate[161]*x_e*n_H-rate[275]*x[49]*n_H-rate[276]*x[38]*n_H-rate[277]*x[36]*n_H-rate[278]*x[36]*n_H-rate[279]*x[46]*n_H-rate[280]*x[51]*n_H-rate[281]*x[55]*n_H-rate[282]*x[42]*n_H-rate[283]*x[18]*n_H-rate[284]*x[18]*n_H-rate[285]*x[54]*n_H-rate[286]*x[52]*n_H-rate[529];
  form = +rate[66]*x[9]*x[55]*n_H+rate[109]*x[55]+rate[537]*x[2]+rate[565]*x[40]*x[56]*n_H+rate[593]*x[55]*zeta_H2;
  ode[14] = form+x[14]*loss;
  loss = -rate[173]*x_e*n_H-rate[174]*x_e*n_H;
  form = +rate[223]*x[18]*x[45]*n_H+rate[236]*x[18]*x[39]*n_H+rate[248]*x[28]*x[36]*n_H+rate[251]*x[36]*x[48]*n_H+rate[320]*x[2]*x[8]*n_H+rate[346]*x[8]*x[41]*n_H;
  ode[15] = form+x[15]*loss;
  loss = -rate[154]*x_e*n_H-rate[155]*x_e*n_H-rate[156]*x_e*n_H-rate[157]*x_e*n_H-rate[158]*x_e*n_H-rate[205]*x[49]*n_H-rate[229]*x[38]*n_H-rate[253]*x[51]*n_H-rate[254]*x[42]*n_H-rate[255]*x[18]*n_H-rate[256]*x[6]*n_H-rate[257]*x[53]*n_H-rate[258]*x[46]*n_H-rate[341]*x[56]*n_H-rate[390]*x[54]*n_H-rate[405]*x[52]*n_H;
  form = +rate[241]*x[19]*x[36]*n_H+rate[249]*x[22]*x[36]*n_H+rate[278]*x[14]*x[36]*n_H+rate[290]*x[19]*x[55]*n_H+rate[316]*x[2]*x[36]*n_H+rate[568]*x[39]*x[55]*n_H;
  ode[16] = form+x[16]*loss;
  loss = -rate[188]*x_e*n_H-rate[189]*x_e*n_H-rate[311]*x[18]*n_H-rate[571]*x[55]*n_H;
  form = +rate[49]*x[5]*x[40]*n_H+rate[330]*x[2]*x[12]*n_H+rate[336]*x[12]*x[33]*n_H+rate[350]*x[12]*x[41]*n_H+rate[354]*x[18]*x[28]*n_H+rate[410]*x[18]*x[48]*n_H+rate[543]*x[5];
  ode[17] = form+x[17]*loss;
  loss = -rate[17]*x[35]*n_H-rate[30]*x[19]*n_H-rate[38]*x[30]*n_H-rate[48]*x[40]*n_H-rate[63]*x[14]*n_H-rate[67]*x[29]*n_H-rate[80]*x[9]*n_H-rate[91]*x[24]*n_H-rate[94]*x[27]*n_H-rate[98]*x[22]*n_H-rate[128]-rate[129]-rate[197]*x[35]*n_H-rate[215]*x[43]*n_H-rate[216]*x[43]*n_H-rate[223]*x[45]*n_H-rate[224]*x[45]*n_H-rate[225]*x[45]*n_H-rate[236]*x[39]*n_H-rate[244]*x[19]*n_H-rate[255]*x[16]*n_H-rate[268]*x[30]*n_H-rate[271]*x[40]*n_H-rate[272]*x[40]*n_H-rate[283]*x[14]*n_H-rate[284]*x[14]*n_H-rate[302]*x[29]*n_H-rate[303]*x[29]*n_H-rate[310]*x[37]*n_H-rate[311]*x[17]*n_H-rate[312]*x[23]*n_H-rate[322]*x[2]*n_H-rate[334]*x[33]*n_H-rate[347]*x[41]*n_H-rate[353]*x[28]*n_H-rate[354]*x[28]*n_H-rate[371]*x[9]*n_H-rate[372]*x[9]*n_H-rate[384]*x[24]*n_H-rate[385]*x[24]*n_H-rate[401]*x[22]*n_H-rate[410]*x[48]*n_H-rate[411]*x[48]*n_H-rate[446]*x[44]*n_H-rate[477]*x[56]*n_H-rate[491]*x[54]*n_H-rate[499]*x[52]*n_H-rate[534]-rate[535]-rate[536]-rate[618]*x[55]*zeta_H2-rate[619]*x[55]*zeta_H2-rate[620]*x[55]*zeta_H2;
  form = +rate[83]*x[6]*x[37]*n_H+rate[104]*x[37]*x[53]*n_H+rate[179]*x[26]*x_e*n_H+rate[416]*x[1]*x[37]*n_H+rate[466]*x[47]*x[55]*n_H+rate[485]*x[47]*x[47]*n_H+rate[583]*x[37]*x_e*n_H;
  ode[18] = form+x[18]*loss;
  loss = -rate[30]*x[18]*n_H-rate[31]*x[21]*n_H-rate[152]*x_e*n_H-rate[153]*x_e*n_H-rate[241]*x[36]*n_H-rate[242]*x[51]*n_H-rate[243]*x[42]*n_H-rate[244]*x[18]*n_H-rate[290]*x[55]*n_H-rate[340]*x[56]*n_H-rate[389]*x[54]*n_H-rate[516]-rate[517];
  form = +rate[32]*x[30]*x[36]*n_H+rate[43]*x[36]*x[40]*n_H+rate[59]*x[14]*x[36]*n_H+rate[77]*x[9]*x[36]*n_H+rate[88]*x[24]*x[36]*n_H+rate[315]*x[2]*x[44]*n_H+rate[341]*x[16]*x[56]*n_H+rate[520]*x[36];
  ode[19] = form+x[19]*loss;
  loss = -rate[19]*x[35]*n_H-rate[57]*x[40]*n_H-rate[143]-rate[144]-rate[202]*x[35]*n_H-rate[203]*x[35]*n_H-rate[204]*x[35]*n_H-rate[332]*x[2]*n_H-rate[352]*x[41]*n_H-rate[381]*x[9]*n_H-rate[382]*x[9]*n_H-rate[432]*x[49]*n_H-rate[433]*x[49]*n_H-rate[461]*x[46]*n_H-rate[483]*x[56]*n_H-rate[484]*x[56]*n_H-rate[497]*x[54]*n_H-rate[503]*x[53]*n_H-rate[557]-rate[558]-rate[616]*x[55]*zeta_H2-rate[617]*x[55]*zeta_H2;
  form = +rate[3]*x[1]*x[54]*n_H+rate[87]*x[6]*x[23]*n_H+rate[190]*x[7]*x_e*n_H+rate[424]*x[1]*x[23]*n_H+rate[486]*x[21]*x[53]*n_H+rate[489]*x[34]*x[54]*n_H+rate[494]*x[47]*x[54]*n_H+rate[496]*x[12]*x[54]*n_H+rate[501]*x[52]*x[53]*n_H;
  ode[20] = form+x[20]*loss;
  loss = -rate[7]*x[55]*n_H-rate[13]*x[56]*n_H-rate[31]*x[19]*n_H-rate[39]*x[30]*n_H-rate[52]*x[40]*n_H-rate[64]*x[14]*n_H-rate[69]*x[29]*n_H-rate[81]*x[9]*n_H-rate[92]*x[24]*n_H-rate[99]*x[22]*n_H-rate[136]-rate[137]-rate[199]*x[35]*n_H-rate[200]*x[35]*n_H-rate[217]*x[43]*n_H-rate[218]*x[43]*n_H-rate[226]*x[45]*n_H-rate[378]*x[9]*n_H-rate[388]*x[48]*n_H-rate[428]*x[49]*n_H-rate[436]*x[38]*n_H-rate[453]*x[46]*n_H-rate[454]*x[46]*n_H-rate[455]*x[46]*n_H-rate[456]*x[46]*n_H-rate[467]*x[55]*n_H-rate[480]*x[56]*n_H-rate[486]*x[53]*n_H-rate[548]-rate[549]-rate[609]*x[55]*zeta_H2-rate[610]*x[55]*zeta_H2-rate[629]*x[56]*zeta_H;
  form = +rate[21]*x[27]*x[49]*n_H+rate[27]*x[27]*x[38]*n_H+rate[36]*x[27]*x[46]*n_H+rate[85]*x[6]*x[27]*n_H+rate[94]*x[18]*x[27]*n_H+rate[95]*x[27]*x[53]*n_H+rate[495]*x[52]*x[54]*n_H+rate[497]*x[20]*x[54]*n_H+rate[577]*x[54]*x[54]*n_H;
  ode[21] = form+x[21]*loss;
  loss = -rate[28]*x[38]*n_H-rate[37]*x[46]*n_H-rate[97]*x[42]*n_H-rate[98]*x[18]*n_H-rate[99]*x[21]*n_H-rate[100]*x[53]*n_H-rate[192]*x_e*n_H-rate[212]*x[49]*n_H-rate[234]*x[38]*n_H-rate[249]*x[36]*n_H-rate[250]*x[36]*n_H-rate[266]*x[46]*n_H-rate[298]*x[55]*n_H-rate[398]*x[54]*n_H-rate[399]*x[51]*n_H-rate[400]*x[42]*n_H-rate[401]*x[18]*n_H-rate[402]*x[52]*n_H-rate[403]*x[53]*n_H-rate[404]*x[53]*n_H-rate[550];
  form = +rate[54]*x[40]*x[52]*n_H+rate[65]*x[14]*x[52]*n_H+rate[93]*x[24]*x[52]*n_H+rate[101]*x[30]*x[52]*n_H+rate[285]*x[14]*x[54]*n_H+rate[297]*x[24]*x[55]*n_H+rate[328]*x[2]*x[54]*n_H+rate[369]*x[9]*x[42]*n_H+rate[531]*x[29]+rate[552]*x[52];
  ode[22] = form+x[22]*loss;
  loss = -rate[87]*x[6]*n_H-rate[194]*x_e*n_H-rate[312]*x[18]*n_H-rate[424]*x[1]*n_H;
  form = +rate[19]*x[20]*x[35]*n_H+rate[57]*x[20]*x[40]*n_H+rate[144]*x[20]+rate[387]*x[27]*x[53]*n_H+rate[388]*x[21]*x[48]*n_H+rate[394]*x[37]*x[54]*n_H+rate[397]*x[28]*x[54]*n_H+rate[404]*x[22]*x[53]*n_H+rate[409]*x[48]*x[52]*n_H+rate[558]*x[20]+rate[617]*x[20]*x[55]*zeta_H2;
  ode[23] = form+x[23]*loss;
  loss = -rate[26]*x[38]*n_H-rate[35]*x[46]*n_H-rate[75]*x[56]*n_H-rate[88]*x[36]*n_H-rate[89]*x[51]*n_H-rate[90]*x[42]*n_H-rate[91]*x[18]*n_H-rate[92]*x[21]*n_H-rate[93]*x[52]*n_H-rate[264]*x[46]*n_H-rate[297]*x[55]*n_H-rate[383]*x[36]*n_H-rate[384]*x[18]*n_H-rate[385]*x[18]*n_H-rate[386]*x[52]*n_H-rate[422]*x[1]*n_H-rate[561]*x[49]*n_H-rate[586]*x_e*n_H;
  form = +rate[53]*x[40]*x[54]*n_H+rate[96]*x[30]*x[54]*n_H+rate[113]*x[54]+rate[138]*x[54]+rate[200]*x[21]*x[35]*n_H+rate[378]*x[9]*x[21]*n_H+rate[379]*x[9]*x[52]*n_H+rate[382]*x[9]*x[20]*n_H+rate[547]*x[27]+rate[550]*x[22]+rate[596]*x[54]*zeta_H;
  ode[24] = form+x[24]*loss;
  loss = -rate[82]*x[6]*n_H-rate[160]*x_e*n_H-rate[246]*x[36]*n_H-rate[292]*x[55]*n_H-rate[391]*x[54]*n_H-rate[526];
  form = +rate[45]*x[34]*x[40]*n_H+rate[124]*x[34]+rate[198]*x[35]*x[47]*n_H+rate[202]*x[20]*x[35]*n_H+rate[210]*x[28]*x[49]*n_H+rate[221]*x[43]*x[53]*n_H+rate[267]*x[46]*x[48]*n_H+rate[273]*x[4]*x[40]*n_H+rate[366]*x[8]*x[9]*n_H+rate[373]*x[4]*x[9]*n_H+rate[527]*x[34]+rate[560]*x[35]*x[53]*n_H+rate[563]*x[48]*x[49]*n_H+rate[626]*x[34]*x[55]*zeta_H2;
  ode[25] = form+x[25]*loss;
  loss = -rate[179]*x_e*n_H-rate[180]*x_e*n_H-rate[181]*x_e*n_H-rate[182]*x_e*n_H-rate[343]*x[56]*n_H-rate[412]*x[53]*n_H;
  form = +rate[215]*x[18]*x[43]*n_H+rate[224]*x[18]*x[45]*n_H+rate[244]*x[18]*x[19]*n_H+rate[255]*x[16]*x[18]*n_H+rate[294]*x[37]*x[55]*n_H+rate[302]*x[18]*x[29]*n_H+rate[310]*x[18]*x[37]*n_H+rate[311]*x[17]*x[18]*n_H+rate[322]*x[2]*x[18]*n_H+rate[334]*x[18]*x[33]*n_H+rate[347]*x[18]*x[41]*n_H+rate[353]*x[18]*x[28]*n_H+rate[401]*x[18]*x[22]*n_H+rate[570]*x[28]*x[55]*n_H;
  ode[26] = form+x[26]*loss;
  loss = -rate[21]*x[49]*n_H-rate[27]*x[38]*n_H-rate[36]*x[46]*n_H-rate[85]*x[6]*n_H-rate[94]*x[18]*n_H-rate[95]*x[53]*n_H-rate[191]*x_e*n_H-rate[211]*x[49]*n_H-rate[265]*x[46]*n_H-rate[387]*x[53]*n_H-rate[547];
  form = +rate[31]*x[19]*x[21]*n_H+rate[39]*x[21]*x[30]*n_H+rate[52]*x[21]*x[40]*n_H+rate[64]*x[14]*x[21]*n_H+rate[69]*x[21]*x[29]*n_H+rate[81]*x[9]*x[21]*n_H+rate[92]*x[21]*x[24]*n_H+rate[99]*x[21]*x[22]*n_H+rate[136]*x[21]+rate[386]*x[24]*x[52]*n_H+rate[392]*x[29]*x[54]*n_H+rate[398]*x[22]*x[54]*n_H+rate[548]*x[21]+rate[610]*x[21]*x[55]*zeta_H2;
  ode[27] = form+x[27]*loss;
  loss = -rate[84]*x[6]*n_H-rate[105]*x[53]*n_H-rate[187]*x_e*n_H-rate[210]*x[49]*n_H-rate[248]*x[36]*n_H-rate[263]*x[46]*n_H-rate[295]*x[55]*n_H-rate[309]*x[42]*n_H-rate[344]*x[56]*n_H-rate[353]*x[18]*n_H-rate[354]*x[18]*n_H-rate[396]*x[54]*n_H-rate[397]*x[54]*n_H-rate[541]-rate[542]-rate[570]*x[55]*n_H;
  form = +rate[50]*x[40]*x[47]*n_H+rate[222]*x[43]*x[53]*n_H+rate[257]*x[16]*x[53]*n_H+rate[271]*x[18]*x[40]*n_H+rate[283]*x[14]*x[18]*n_H+rate[299]*x[48]*x[55]*n_H+rate[304]*x[29]*x[53]*n_H+rate[331]*x[2]*x[53]*n_H+rate[342]*x[37]*x[56]*n_H+rate[351]*x[41]*x[53]*n_H+rate[371]*x[9]*x[18]*n_H+rate[384]*x[18]*x[24]*n_H+rate[393]*x[37]*x[54]*n_H+rate[403]*x[22]*x[53]*n_H;
  ode[28] = form+x[28]*loss;
  loss = -rate[25]*x[38]*n_H-rate[34]*x[46]*n_H-rate[67]*x[18]*n_H-rate[68]*x[6]*n_H-rate[69]*x[21]*n_H-rate[70]*x[53]*n_H-rate[164]*x_e*n_H-rate[165]*x_e*n_H-rate[166]*x_e*n_H-rate[206]*x[49]*n_H-rate[231]*x[38]*n_H-rate[247]*x[36]*n_H-rate[260]*x[46]*n_H-rate[293]*x[55]*n_H-rate[300]*x[51]*n_H-rate[301]*x[42]*n_H-rate[302]*x[18]*n_H-rate[303]*x[18]*n_H-rate[304]*x[53]*n_H-rate[305]*x[53]*n_H-rate[392]*x[54]*n_H-rate[407]*x[52]*n_H-rate[531];
  form = +rate[47]*x[40]*x[42]*n_H+rate[62]*x[14]*x[42]*n_H+rate[71]*x[30]*x[42]*n_H+rate[79]*x[9]*x[42]*n_H+rate[90]*x[24]*x[42]*n_H+rate[97]*x[22]*x[42]*n_H+rate[286]*x[14]*x[52]*n_H+rate[298]*x[22]*x[55]*n_H+rate[327]*x[2]*x[54]*n_H+rate[329]*x[2]*x[52]*n_H+rate[402]*x[22]*x[52]*n_H+rate[405]*x[16]*x[52]*n_H+rate[408]*x[41]*x[52]*n_H+rate[532]*x[42];
  ode[29] = form+x[29]*loss;
  loss = -rate[20]*x[49]*n_H-rate[24]*x[38]*n_H-rate[32]*x[36]*n_H-rate[33]*x[46]*n_H-rate[38]*x[18]*n_H-rate[39]*x[21]*n_H-rate[40]*x[53]*n_H-rate[71]*x[42]*n_H-rate[72]*x[56]*n_H-rate[96]*x[54]*n_H-rate[101]*x[52]*n_H-rate[159]*x_e*n_H-rate[230]*x[38]*n_H-rate[245]*x[36]*n_H-rate[259]*x[46]*n_H-rate[268]*x[18]*n_H-rate[291]*x[55]*n_H-rate[306]*x[42]*n_H-rate[406]*x[52]*n_H-rate[524];
  form = +rate[61]*x[14]*x[51]*n_H+rate[89]*x[24]*x[51]*n_H+rate[107]*x[51]+rate[199]*x[21]*x[35]*n_H+rate[201]*x[35]*x[52]*n_H+rate[204]*x[20]*x[35]*n_H+rate[211]*x[27]*x[49]*n_H+rate[217]*x[21]*x[43]*n_H+rate[219]*x[43]*x[54]*n_H+rate[220]*x[43]*x[52]*n_H+rate[264]*x[24]*x[46]*n_H+rate[391]*x[25]*x[54]*n_H+rate[539]*x[41]+rate[559]*x[35]*x[54]*n_H+rate[561]*x[24]*x[49]*n_H;
  ode[30] = form+x[30]*loss;
  loss = -rate[421]*x[1]*n_H-rate[585]*x_e*n_H;
  form = +rate[18]*x[6]*x[35]*n_H+rate[22]*x[6]*x[43]*n_H+rate[29]*x[6]*x[39]*n_H+rate[51]*x[6]*x[40]*n_H+rate[68]*x[6]*x[29]*n_H+rate[82]*x[6]*x[25]*n_H+rate[83]*x[6]*x[37]*n_H+rate[84]*x[6]*x[28]*n_H+rate[85]*x[6]*x[27]*n_H+rate[86]*x[6]*x[48]*n_H+rate[87]*x[6]*x[23]*n_H+rate[135]*x[6]+rate[256]*x[6]*x[16]*n_H+rate[326]*x[2]*x[6]*n_H+rate[546]*x[6]+rate[627]*x[6]*x[55]*zeta_H2;
  ode[31] = form+x[31]*loss;
  loss = -rate[185]*x_e*n_H-rate[186]*x_e*n_H-rate[395]*x[54]*n_H;
  form = +rate[131]*x[4]+rate[197]*x[18]*x[35]*n_H+rate[207]*x[37]*x[49]*n_H+rate[216]*x[18]*x[43]*n_H+rate[225]*x[18]*x[45]*n_H+rate[228]*x[45]*x[53]*n_H+rate[235]*x[38]*x[48]*n_H+rate[239]*x[39]*x[53]*n_H+rate[246]*x[25]*x[36]*n_H+rate[252]*x[36]*x[48]*n_H+rate[292]*x[25]*x[55]*n_H+rate[319]*x[2]*x[34]*n_H+rate[333]*x[33]*x[34]*n_H+rate[345]*x[34]*x[41]*n_H+rate[540]*x[4]+rate[621]*x[4]*x[55]*zeta_H2;
  ode[32] = form+x[32]*loss;
  loss = -rate[175]*x_e*n_H-rate[176]*x_e*n_H-rate[177]*x_e*n_H-rate[178]*x_e*n_H-rate[208]*x[49]*n_H-rate[232]*x[38]*n_H-rate[261]*x[46]*n_H-rate[333]*x[34]*n_H-rate[334]*x[18]*n_H-rate[335]*x[5]*n_H-rate[336]*x[12]*n_H-rate[418]*x[1]*n_H;
  form = +rate[213]*x[42]*x[43]*n_H+rate[243]*x[19]*x[42]*n_H+rate[247]*x[29]*x[36]*n_H+rate[250]*x[22]*x[36]*n_H+rate[254]*x[16]*x[42]*n_H+rate[282]*x[14]*x[42]*n_H+rate[293]*x[29]*x[55]*n_H+rate[301]*x[29]*x[42]*n_H+rate[303]*x[18]*x[29]*n_H+rate[307]*x[37]*x[42]*n_H+rate[308]*x[41]*x[42]*n_H+rate[309]*x[28]*x[42]*n_H+rate[321]*x[2]*x[42]*n_H+rate[390]*x[16]*x[54]*n_H+rate[400]*x[22]*x[42]*n_H+rate[407]*x[29]*x[52]*n_H;
  ode[33] = form+x[33]*loss;
  loss = -rate[45]*x[40]*n_H-rate[124]-rate[125]-rate[319]*x[2]*n_H-rate[333]*x[33]*n_H-rate[345]*x[41]*n_H-rate[364]*x[9]*n_H-rate[365]*x[9]*n_H-rate[488]*x[54]*n_H-rate[489]*x[54]*n_H-rate[498]*x[52]*n_H-rate[527]-rate[528]-rate[625]*x[55]*zeta_H2-rate[626]*x[55]*zeta_H2;
  form = +rate[0]*x[1]*x[49]*n_H+rate[82]*x[6]*x[25]*n_H+rate[126]*x[8]+rate[162]*x[11]*x_e*n_H+rate[173]*x[15]*x_e*n_H+rate[186]*x[32]*x_e*n_H+rate[195]*x[8]*x[35]*n_H+rate[374]*x[4]*x[9]*n_H+rate[426]*x[47]*x[49]*n_H+rate[431]*x[12]*x[49]*n_H+rate[432]*x[20]*x[49]*n_H+rate[442]*x[38]*x[53]*n_H+rate[459]*x[46]*x[53]*n_H+rate[478]*x[4]*x[56]*n_H+rate[530]*x[8]+rate[564]*x[49]*x[53]*n_H+rate[623]*x[8]*x[55]*zeta_H2;
  ode[34] = form+x[34]*loss;
  loss = -rate[15]*x[38]*n_H-rate[16]*x[46]*n_H-rate[17]*x[18]*n_H-rate[18]*x[6]*n_H-rate[19]*x[20]*n_H-rate[102]*x[53]*n_H-rate[103]*x[53]*n_H-rate[195]*x[8]*n_H-rate[196]*x[42]*n_H-rate[197]*x[18]*n_H-rate[198]*x[47]*n_H-rate[199]*x[21]*n_H-rate[200]*x[21]*n_H-rate[201]*x[52]*n_H-rate[202]*x[20]*n_H-rate[203]*x[20]*n_H-rate[204]*x[20]*n_H-rate[287]*x[55]*n_H-rate[413]*x[1]*n_H-rate[559]*x[54]*n_H-rate[560]*x[53]*n_H-rate[566]*x[55]*n_H-rate[573]*x[56]*n_H-rate[579]*x_e*n_H;
  form = +rate[20]*x[30]*x[49]*n_H+rate[21]*x[27]*x[49]*n_H+rate[76]*x[9]*x[49]*n_H+rate[106]*x[49]+rate[114]*x[49]+rate[115]*x[43]+rate[337]*x[43]*x[56]*n_H+rate[355]*x[9]*x[38]*n_H+rate[362]*x[9]*x[46]*n_H+rate[363]*x[9]*x[51]*n_H+rate[365]*x[9]*x[34]*n_H+rate[504]*x[49]+rate[506]*x[45]+rate[524]*x[30]+rate[595]*x[49]*zeta_H+rate[598]*x[49]*x[55]*zeta_H2+rate[600]*x[43]*x[55]*zeta_H2;
  ode[35] = form+x[35]*loss;
  loss = -rate[32]*x[30]*n_H-rate[43]*x[40]*n_H-rate[59]*x[14]*n_H-rate[77]*x[9]*n_H-rate[88]*x[24]*n_H-rate[121]-rate[241]*x[19]*n_H-rate[245]*x[30]*n_H-rate[246]*x[25]*n_H-rate[247]*x[29]*n_H-rate[248]*x[28]*n_H-rate[249]*x[22]*n_H-rate[250]*x[22]*n_H-rate[251]*x[48]*n_H-rate[252]*x[48]*n_H-rate[270]*x[40]*n_H-rate[277]*x[14]*n_H-rate[278]*x[14]*n_H-rate[316]*x[2]*n_H-rate[358]*x[9]*n_H-rate[359]*x[9]*n_H-rate[360]*x[9]*n_H-rate[361]*x[9]*n_H-rate[383]*x[24]*n_H-rate[435]*x[38]*n_H-rate[451]*x[52]*n_H-rate[452]*x[53]*n_H-rate[473]*x[56]*n_H-rate[487]*x[54]*n_H-rate[518]-rate[519]-rate[520]-rate[521]-rate[606]*x[55]*zeta_H2-rate[607]*x[55]*zeta_H2-rate[631]*x[56]*zeta_H;
  form = +rate[30]*x[18]*x[19]*n_H+rate[31]*x[19]*x[21]*n_H+rate[157]*x[16]*x_e*n_H+rate[205]*x[16]*x[49]*n_H+rate[229]*x[16]*x[38]*n_H+rate[253]*x[16]*x[51]*n_H+rate[254]*x[16]*x[42]*n_H+rate[255]*x[16]*x[18]*n_H+rate[256]*x[6]*x[16]*n_H+rate[257]*x[16]*x[53]*n_H+rate[258]*x[16]*x[46]*n_H+rate[405]*x[16]*x[52]*n_H+rate[444]*x[44]*x[44]*n_H+rate[445]*x[42]*x[44]*n_H+rate[446]*x[18]*x[44]*n_H+rate[448]*x[44]*x[52]*n_H+rate[464]*x[44]*x[55]*n_H;
  ode[36] = form+x[36]*loss;
  loss = -rate[83]*x[6]*n_H-rate[104]*x[53]*n_H-rate[167]*x_e*n_H-rate[168]*x_e*n_H-rate[207]*x[49]*n_H-rate[294]*x[55]*n_H-rate[307]*x[42]*n_H-rate[310]*x[18]*n_H-rate[342]*x[56]*n_H-rate[393]*x[54]*n_H-rate[394]*x[54]*n_H-rate[416]*x[1]*n_H-rate[583]*x_e*n_H;
  form = +rate[17]*x[18]*x[35]*n_H+rate[30]*x[18]*x[19]*n_H+rate[38]*x[18]*x[30]*n_H+rate[48]*x[18]*x[40]*n_H+rate[63]*x[14]*x[18]*n_H+rate[67]*x[18]*x[29]*n_H+rate[80]*x[9]*x[18]*n_H+rate[91]*x[18]*x[24]*n_H+rate[94]*x[18]*x[27]*n_H+rate[98]*x[18]*x[22]*n_H+rate[128]*x[18]+rate[295]*x[28]*x[55]*n_H+rate[325]*x[2]*x[47]*n_H+rate[343]*x[26]*x[56]*n_H+rate[349]*x[41]*x[47]*n_H+rate[534]*x[18]+rate[572]*x[48]*x[55]*n_H+rate[619]*x[18]*x[55]*zeta_H2;
  ode[37] = form+x[37]*loss;
  loss = -rate[15]*x[35]*n_H-rate[24]*x[30]*n_H-rate[25]*x[29]*n_H-rate[26]*x[24]*n_H-rate[27]*x[27]*n_H-rate[28]*x[22]*n_H-rate[41]*x[40]*n_H-rate[58]*x[14]*n_H-rate[116]-rate[117]-rate[229]*x[16]*n_H-rate[230]*x[30]*n_H-rate[231]*x[29]*n_H-rate[232]*x[33]*n_H-rate[233]*x[41]*n_H-rate[234]*x[22]*n_H-rate[235]*x[48]*n_H-rate[269]*x[40]*n_H-rate[276]*x[14]*n_H-rate[314]*x[2]*n_H-rate[355]*x[9]*n_H-rate[356]*x[9]*n_H-rate[425]*x[49]*n_H-2*rate[434]*x[38]*n_H-rate[435]*x[36]*n_H-rate[436]*x[21]*n_H-rate[437]*x[54]*n_H-rate[438]*x[54]*n_H-rate[439]*x[54]*n_H-rate[440]*x[52]*n_H-rate[441]*x[52]*n_H-rate[442]*x[53]*n_H-rate[443]*x[53]*n_H-rate[463]*x[55]*n_H-rate[471]*x[56]*n_H-rate[509]-rate[510]-rate[601]*x[55]*zeta_H2-rate[602]*x[55]*zeta_H2;
  form = +rate[118]*x[44]+rate[121]*x[36]+rate[149]*x[39]*x_e*n_H+rate[152]*x[19]*x_e*n_H+rate[154]*x[16]*x_e*n_H+rate[250]*x[22]*x[36]*n_H+rate[367]*x[8]*x[9]*n_H+rate[390]*x[16]*x[54]*n_H+rate[444]*x[44]*x[44]*n_H+rate[449]*x[44]*x[52]*n_H+rate[465]*x[46]*x[55]*n_H+rate[472]*x[44]*x[56]*n_H+rate[513]*x[44]+rate[518]*x[36]+rate[567]*x[49]*x[55]*n_H+rate[603]*x[44]*x[55]*zeta_H2+rate[606]*x[36]*x[55]*zeta_H2+rate[631]*x[36]*x[56]*zeta_H;
  ode[38] = form+x[38]*loss;
  loss = -rate[29]*x[6]*n_H-rate[149]*x_e*n_H-rate[150]*x_e*n_H-rate[151]*x_e*n_H-rate[236]*x[18]*n_H-rate[237]*x[47]*n_H-rate[238]*x[54]*n_H-rate[239]*x[53]*n_H-rate[339]*x[56]*n_H-rate[414]*x[1]*n_H-rate[511]-rate[512]-rate[568]*x[55]*n_H-rate[580]*x_e*n_H;
  form = +rate[42]*x[40]*x[44]*n_H+rate[119]*x[44]+rate[229]*x[16]*x[38]*n_H+rate[231]*x[29]*x[38]*n_H+rate[232]*x[33]*x[38]*n_H+rate[233]*x[38]*x[41]*n_H+rate[234]*x[22]*x[38]*n_H+rate[270]*x[36]*x[40]*n_H+rate[276]*x[14]*x[38]*n_H+rate[277]*x[14]*x[36]*n_H+rate[289]*x[45]*x[55]*n_H+rate[314]*x[2]*x[38]*n_H+rate[340]*x[19]*x[56]*n_H+rate[360]*x[9]*x[36]*n_H+rate[383]*x[24]*x[36]*n_H+rate[389]*x[19]*x[54]*n_H+rate[514]*x[44]+rate[517]*x[19]+rate[604]*x[44]*x[55]*zeta_H2;
  ode[39] = form+x[39]*loss;
  loss = -rate[41]*x[38]*n_H-rate[42]*x[44]*n_H-rate[43]*x[36]*n_H-rate[44]*x[46]*n_H-rate[45]*x[34]*n_H-rate[46]*x[8]*n_H-rate[47]*x[42]*n_H-rate[48]*x[18]*n_H-rate[49]*x[5]*n_H-rate[50]*x[47]*n_H-rate[51]*x[6]*n_H-rate[52]*x[21]*n_H-rate[53]*x[54]*n_H-rate[54]*x[52]*n_H-rate[55]*x[12]*n_H-rate[56]*x[53]*n_H-rate[57]*x[20]*n_H-rate[269]*x[38]*n_H-rate[270]*x[36]*n_H-rate[271]*x[18]*n_H-rate[272]*x[18]*n_H-rate[273]*x[4]*n_H-rate[274]*x[47]*n_H-rate[415]*x[1]*n_H-rate[565]*x[56]*n_H-rate[581]*x_e*n_H;
  form = +rate[72]*x[30]*x[56]*n_H+rate[73]*x[14]*x[56]*n_H+rate[74]*x[9]*x[56]*n_H+rate[75]*x[24]*x[56]*n_H+rate[108]*x[55]+rate[111]*x[56]+rate[130]*x[56]+rate[296]*x[9]*x[55]*n_H+rate[361]*x[9]*x[36]*n_H+rate[370]*x[9]*x[42]*n_H+rate[374]*x[4]*x[9]*n_H+rate[505]*x[43]+rate[508]*x[45]+rate[529]*x[14]+rate[538]*x[2]+rate[542]*x[28]+rate[592]*x[56]*zeta_H+rate[599]*x[43]*x[55]*zeta_H2+rate[633]*x[3]*x[56]*n_H;
  ode[40] = form+x[40]*loss;
  loss = -rate[184]*x_e*n_H-rate[209]*x[49]*n_H-rate[233]*x[38]*n_H-rate[262]*x[46]*n_H-rate[308]*x[42]*n_H-rate[345]*x[34]*n_H-rate[346]*x[8]*n_H-rate[347]*x[18]*n_H-rate[348]*x[5]*n_H-rate[349]*x[47]*n_H-rate[350]*x[12]*n_H-rate[351]*x[53]*n_H-rate[352]*x[20]*n_H-rate[408]*x[52]*n_H-rate[419]*x[1]*n_H-rate[539];
  form = +rate[1]*x[46]*x[54]*n_H+rate[196]*x[35]*x[42]*n_H+rate[208]*x[33]*x[49]*n_H+rate[214]*x[42]*x[43]*n_H+rate[218]*x[21]*x[43]*n_H+rate[226]*x[21]*x[45]*n_H+rate[227]*x[45]*x[54]*n_H+rate[230]*x[30]*x[38]*n_H+rate[238]*x[39]*x[54]*n_H+rate[242]*x[19]*x[51]*n_H+rate[245]*x[30]*x[36]*n_H+rate[253]*x[16]*x[51]*n_H+rate[259]*x[30]*x[46]*n_H+rate[265]*x[27]*x[46]*n_H+rate[268]*x[18]*x[30]*n_H+rate[280]*x[14]*x[51]*n_H+rate[291]*x[30]*x[55]*n_H+rate[300]*x[29]*x[51]*n_H+rate[306]*x[30]*x[42]*n_H+rate[318]*x[2]*x[51]*n_H+rate[395]*x[32]*x[54]*n_H+rate[399]*x[22]*x[51]*n_H+rate[406]*x[30]*x[52]*n_H;
  ode[41] = form+x[41]*loss;
  loss = -rate[6]*x[55]*n_H-rate[12]*x[56]*n_H-rate[47]*x[40]*n_H-rate[62]*x[14]*n_H-rate[71]*x[30]*n_H-rate[79]*x[9]*n_H-rate[90]*x[24]*n_H-rate[97]*x[22]*n_H-rate[127]-rate[196]*x[35]*n_H-rate[213]*x[43]*n_H-rate[214]*x[43]*n_H-rate[243]*x[19]*n_H-rate[254]*x[16]*n_H-rate[282]*x[14]*n_H-rate[301]*x[29]*n_H-rate[306]*x[30]*n_H-rate[307]*x[37]*n_H-rate[308]*x[41]*n_H-rate[309]*x[28]*n_H-rate[321]*x[2]*n_H-rate[369]*x[9]*n_H-rate[370]*x[9]*n_H-rate[400]*x[22]*n_H-rate[445]*x[44]*n_H-rate[476]*x[56]*n_H-rate[490]*x[54]*n_H-rate[532]-rate[533]-rate[612]*x[55]*zeta_H2-rate[630]*x[56]*zeta_H;
  form = +rate[25]*x[29]*x[38]*n_H+rate[34]*x[29]*x[46]*n_H+rate[67]*x[18]*x[29]*n_H+rate[68]*x[6]*x[29]*n_H+rate[69]*x[21]*x[29]*n_H+rate[70]*x[29]*x[53]*n_H+rate[175]*x[33]*x_e*n_H+rate[232]*x[33]*x[38]*n_H+rate[261]*x[33]*x[46]*n_H+rate[312]*x[18]*x[23]*n_H+rate[333]*x[33]*x[34]*n_H+rate[334]*x[18]*x[33]*n_H+rate[335]*x[5]*x[33]*n_H+rate[336]*x[12]*x[33]*n_H+rate[385]*x[18]*x[24]*n_H+rate[418]*x[1]*x[33]*n_H+rate[436]*x[21]*x[38]*n_H+rate[440]*x[38]*x[52]*n_H+rate[449]*x[44]*x[52]*n_H+rate[451]*x[36]*x[52]*n_H+rate[469]*x[52]*x[55]*n_H+rate[499]*x[18]*x[52]*n_H+rate[500]*x[52]*x[52]*n_H+rate[576]*x[52]*x[56]*n_H;
  ode[42] = form+x[42]*loss;
  loss = -rate[22]*x[6]*n_H-rate[23]*x[53]*n_H-rate[115]-rate[145]*x_e*n_H-rate[213]*x[42]*n_H-rate[214]*x[42]*n_H-rate[215]*x[18]*n_H-rate[216]*x[18]*n_H-rate[217]*x[21]*n_H-rate[218]*x[21]*n_H-rate[219]*x[54]*n_H-rate[220]*x[52]*n_H-rate[221]*x[53]*n_H-rate[222]*x[53]*n_H-rate[288]*x[55]*n_H-rate[337]*x[56]*n_H-rate[505]-rate[599]*x[55]*zeta_H2-rate[600]*x[55]*zeta_H2;
  form = +rate[16]*x[35]*x[46]*n_H+rate[33]*x[30]*x[46]*n_H+rate[34]*x[29]*x[46]*n_H+rate[35]*x[24]*x[46]*n_H+rate[36]*x[27]*x[46]*n_H+rate[37]*x[22]*x[46]*n_H+rate[44]*x[40]*x[46]*n_H+rate[60]*x[14]*x[46]*n_H+rate[78]*x[9]*x[46]*n_H+rate[205]*x[16]*x[49]*n_H+rate[206]*x[29]*x[49]*n_H+rate[209]*x[41]*x[49]*n_H+rate[212]*x[22]*x[49]*n_H+rate[269]*x[38]*x[40]*n_H+rate[275]*x[14]*x[49]*n_H+rate[287]*x[35]*x[55]*n_H+rate[313]*x[2]*x[49]*n_H+rate[338]*x[45]*x[56]*n_H+rate[356]*x[9]*x[38]*n_H+rate[357]*x[9]*x[44]*n_H+rate[358]*x[9]*x[36]*n_H+rate[507]*x[45]+rate[511]*x[39]+rate[523]*x[46]+rate[573]*x[35]*x[56]*n_H;
  ode[43] = form+x[43]*loss;
  loss = -rate[42]*x[40]*n_H-rate[118]-rate[119]-rate[120]-rate[240]*x[48]*n_H-rate[315]*x[2]*n_H-rate[357]*x[9]*n_H-2*rate[444]*x[44]*n_H-rate[445]*x[42]*n_H-rate[446]*x[18]*n_H-rate[447]*x[54]*n_H-rate[448]*x[52]*n_H-rate[449]*x[52]*n_H-rate[450]*x[53]*n_H-rate[464]*x[55]*n_H-rate[472]*x[56]*n_H-rate[513]-rate[514]-rate[515]-rate[603]*x[55]*zeta_H2-rate[604]*x[55]*zeta_H2-rate[605]*x[55]*zeta_H2;
  form = +rate[29]*x[6]*x[39]*n_H+rate[153]*x[19]*x_e*n_H+rate[155]*x[16]*x_e*n_H+rate[156]*x[16]*x_e*n_H+rate[241]*x[19]*x[36]*n_H+rate[242]*x[19]*x[51]*n_H+rate[243]*x[19]*x[42]*n_H+rate[244]*x[18]*x[19]*n_H+rate[245]*x[30]*x[36]*n_H+rate[246]*x[25]*x[36]*n_H+rate[247]*x[29]*x[36]*n_H+rate[361]*x[9]*x[36]*n_H+rate[414]*x[1]*x[39]*n_H+rate[434]*x[38]*x[38]*n_H+2*rate[435]*x[36]*x[38]*n_H+rate[441]*x[38]*x[52]*n_H+rate[451]*x[36]*x[52]*n_H+rate[452]*x[36]*x[53]*n_H+rate[463]*x[38]*x[55]*n_H+rate[473]*x[36]*x[56]*n_H+rate[487]*x[36]*x[54]*n_H+rate[519]*x[36]+rate[569]*x[46]*x[55]*n_H+rate[580]*x[39]*x_e*n_H+rate[607]*x[36]*x[55]*zeta_H2;
  ode[44] = form+x[44]*loss;
  loss = -rate[146]*x_e*n_H-rate[147]*x_e*n_H-rate[148]*x_e*n_H-rate[223]*x[18]*n_H-rate[224]*x[18]*n_H-rate[225]*x[18]*n_H-rate[226]*x[21]*n_H-rate[227]*x[54]*n_H-rate[228]*x[53]*n_H-rate[289]*x[55]*n_H-rate[338]*x[56]*n_H-rate[506]-rate[507]-rate[508];
  form = +rate[15]*x[35]*x[38]*n_H+rate[24]*x[30]*x[38]*n_H+rate[25]*x[29]*x[38]*n_H+rate[26]*x[24]*x[38]*n_H+rate[27]*x[27]*x[38]*n_H+rate[28]*x[22]*x[38]*n_H+rate[41]*x[38]*x[40]*n_H+rate[58]*x[14]*x[38]*n_H+rate[116]*x[38]+rate[195]*x[8]*x[35]*n_H+rate[258]*x[16]*x[46]*n_H+rate[260]*x[29]*x[46]*n_H+rate[261]*x[33]*x[46]*n_H+rate[262]*x[41]*x[46]*n_H+rate[263]*x[28]*x[46]*n_H+rate[266]*x[22]*x[46]*n_H+rate[279]*x[14]*x[46]*n_H+rate[288]*x[43]*x[55]*n_H+rate[317]*x[2]*x[46]*n_H+rate[339]*x[39]*x[56]*n_H+rate[359]*x[9]*x[36]*n_H+rate[368]*x[8]*x[9]*n_H+rate[509]*x[38]+rate[512]*x[39]+rate[516]*x[19]+rate[566]*x[35]*x[55]*n_H+rate[601]*x[38]*x[55]*zeta_H2;
  ode[45] = form+x[45]*loss;
  loss = -rate[1]*x[54]*n_H-rate[4]*x[55]*n_H-rate[10]*x[56]*n_H-rate[16]*x[35]*n_H-rate[33]*x[30]*n_H-rate[34]*x[29]*n_H-rate[35]*x[24]*n_H-rate[36]*x[27]*n_H-rate[37]*x[22]*n_H-rate[44]*x[40]*n_H-rate[60]*x[14]*n_H-rate[78]*x[9]*n_H-rate[122]-rate[258]*x[16]*n_H-rate[259]*x[30]*n_H-rate[260]*x[29]*n_H-rate[261]*x[33]*n_H-rate[262]*x[41]*n_H-rate[263]*x[28]*n_H-rate[264]*x[24]*n_H-rate[265]*x[27]*n_H-rate[266]*x[22]*n_H-rate[267]*x[48]*n_H-rate[279]*x[14]*n_H-rate[317]*x[2]*n_H-rate[362]*x[9]*n_H-rate[453]*x[21]*n_H-rate[454]*x[21]*n_H-rate[455]*x[21]*n_H-rate[456]*x[21]*n_H-rate[457]*x[54]*n_H-rate[458]*x[54]*n_H-rate[459]*x[53]*n_H-rate[460]*x[53]*n_H-rate[461]*x[20]*n_H-rate[465]*x[55]*n_H-rate[474]*x[56]*n_H-rate[522]-rate[523]-rate[569]*x[55]*n_H-rate[608]*x[55]*zeta_H2;
  form = +rate[22]*x[6]*x[43]*n_H+rate[23]*x[43]*x[53]*n_H+rate[117]*x[38]+rate[120]*x[44]+rate[148]*x[45]*x_e*n_H+rate[150]*x[39]*x_e*n_H+rate[151]*x[39]*x_e*n_H+rate[158]*x[16]*x_e*n_H+rate[185]*x[32]*x_e*n_H+rate[224]*x[18]*x[45]*n_H+rate[230]*x[30]*x[38]*n_H+2*rate[425]*x[38]*x[49]*n_H+rate[427]*x[47]*x[49]*n_H+rate[430]*x[49]*x[52]*n_H+rate[434]*x[38]*x[38]*n_H+rate[439]*x[38]*x[54]*n_H+rate[440]*x[38]*x[52]*n_H+rate[462]*x[49]*x[55]*n_H+rate[471]*x[38]*x[56]*n_H+rate[508]*x[45]+rate[510]*x[38]+rate[515]*x[44]+rate[521]*x[36]+rate[574]*x[49]*x[56]*n_H+rate[602]*x[38]*x[55]*zeta_H2+rate[605]*x[44]*x[55]*zeta_H2;
  ode[46] = form+x[46]*loss;
  loss = -rate[50]*x[40]*n_H-rate[133]-rate[198]*x[35]*n_H-rate[237]*x[39]*n_H-rate[274]*x[40]*n_H-rate[325]*x[2]*n_H-rate[349]*x[41]*n_H-rate[377]*x[9]*n_H-rate[426]*x[49]*n_H-rate[427]*x[49]*n_H-rate[466]*x[55]*n_H-rate[479]*x[56]*n_H-2*rate[485]*x[47]*n_H-rate[493]*x[54]*n_H-rate[494]*x[54]*n_H-rate[502]*x[53]*n_H-rate[545]-rate[614]*x[55]*zeta_H2;
  form = +rate[2]*x[1]*x[56]*n_H+rate[84]*x[6]*x[28]*n_H+rate[105]*x[28]*x[53]*n_H+rate[132]*x[5]+rate[167]*x[37]*x_e*n_H+2*rate[170]*x[10]*x_e*n_H+rate[180]*x[26]*x_e*n_H+rate[181]*x[26]*x_e*n_H+rate[188]*x[17]*x_e*n_H+rate[268]*x[18]*x[30]*n_H+rate[303]*x[18]*x[29]*n_H+rate[307]*x[37]*x[42]*n_H+rate[310]*x[18]*x[37]*n_H+rate[375]*x[5]*x[9]*n_H+rate[446]*x[18]*x[44]*n_H+rate[452]*x[36]*x[53]*n_H+rate[460]*x[46]*x[53]*n_H+rate[461]*x[20]*x[46]*n_H+rate[470]*x[53]*x[55]*n_H+rate[477]*x[18]*x[56]*n_H+rate[482]*x[12]*x[56]*n_H+rate[483]*x[20]*x[56]*n_H+rate[491]*x[18]*x[54]*n_H+rate[492]*x[4]*x[54]*n_H+rate[498]*x[34]*x[52]*n_H+rate[499]*x[18]*x[52]*n_H+rate[535]*x[18]+rate[544]*x[5]+rate[620]*x[18]*x[55]*zeta_H2+rate[622]*x[5]*x[55]*zeta_H2;
  ode[47] = form+x[47]*loss;
  loss = -rate[86]*x[6]*n_H-rate[235]*x[38]*n_H-rate[240]*x[44]*n_H-rate[251]*x[36]*n_H-rate[252]*x[36]*n_H-rate[267]*x[46]*n_H-rate[299]*x[55]*n_H-rate[388]*x[21]*n_H-rate[409]*x[52]*n_H-rate[410]*x[18]*n_H-rate[411]*x[18]*n_H-rate[423]*x[1]*n_H-rate[563]*x[49]*n_H-rate[572]*x[55]*n_H-rate[587]*x_e*n_H-rate[591];
  form = +rate[23]*x[43]*x[53]*n_H+rate[40]*x[30]*x[53]*n_H+rate[56]*x[40]*x[53]*n_H+rate[70]*x[29]*x[53]*n_H+rate[95]*x[27]*x[53]*n_H+rate[100]*x[22]*x[53]*n_H+rate[102]*x[35]*x[53]*n_H+rate[103]*x[35]*x[53]*n_H+rate[104]*x[37]*x[53]*n_H+rate[105]*x[28]*x[53]*n_H+rate[142]*x[53]+rate[203]*x[20]*x[35]*n_H+rate[272]*x[18]*x[40]*n_H+rate[274]*x[40]*x[47]*n_H+rate[284]*x[14]*x[18]*n_H+rate[344]*x[28]*x[56]*n_H+rate[364]*x[9]*x[34]*n_H+rate[367]*x[8]*x[9]*n_H+rate[372]*x[9]*x[18]*n_H+rate[375]*x[5]*x[9]*n_H+rate[377]*x[9]*x[47]*n_H+rate[380]*x[9]*x[12]*n_H+rate[381]*x[9]*x[20]*n_H+rate[385]*x[18]*x[24]*n_H+rate[396]*x[28]*x[54]*n_H+rate[526]*x[25]+rate[541]*x[28]+rate[556]*x[53]+rate[597]*x[53]*zeta_H+rate[624]*x[53]*x[55]*zeta_H2+rate[632]*x[3]*x_e*n_H+rate[633]*x[3]*x[56]*n_H;
  ode[48] = form+x[48]*loss;
  loss = -rate[0]*x[1]*n_H-rate[20]*x[30]*n_H-rate[21]*x[27]*n_H-rate[76]*x[9]*n_H-rate[106]-rate[114]-rate[205]*x[16]*n_H-rate[206]*x[29]*n_H-rate[207]*x[37]*n_H-rate[208]*x[33]*n_H-rate[209]*x[41]*n_H-rate[210]*x[28]*n_H-rate[211]*x[27]*n_H-rate[212]*x[22]*n_H-rate[275]*x[14]*n_H-rate[313]*x[2]*n_H-rate[425]*x[38]*n_H-rate[426]*x[47]*n_H-rate[427]*x[47]*n_H-rate[428]*x[21]*n_H-rate[429]*x[52]*n_H-rate[430]*x[52]*n_H-rate[431]*x[12]*n_H-rate[432]*x[20]*n_H-rate[433]*x[20]*n_H-rate[462]*x[55]*n_H-rate[504]-rate[561]*x[24]*n_H-rate[562]*x[54]*n_H-rate[563]*x[48]*n_H-rate[564]*x[53]*n_H-rate[567]*x[55]*n_H-rate[574]*x[56]*n_H-rate[595]*zeta_H-rate[598]*x[55]*zeta_H2;
  form = +rate[4]*x[46]*x[55]*n_H+rate[10]*x[46]*x[56]*n_H+rate[15]*x[35]*x[38]*n_H+rate[16]*x[35]*x[46]*n_H+rate[17]*x[18]*x[35]*n_H+rate[18]*x[6]*x[35]*n_H+rate[19]*x[20]*x[35]*n_H+rate[102]*x[35]*x[53]*n_H+rate[103]*x[35]*x[53]*n_H+rate[122]*x[46]+rate[123]*x[51]+rate[125]*x[34]+rate[145]*x[43]*x_e*n_H+rate[146]*x[45]*x_e*n_H+rate[147]*x[45]*x_e*n_H+rate[159]*x[30]*x_e*n_H+rate[160]*x[25]*x_e*n_H+rate[213]*x[42]*x[43]*n_H+rate[215]*x[18]*x[43]*n_H+rate[222]*x[43]*x[53]*n_H+rate[259]*x[30]*x[46]*n_H+rate[364]*x[9]*x[34]*n_H+rate[413]*x[1]*x[35]*n_H+rate[458]*x[46]*x[54]*n_H+rate[460]*x[46]*x[53]*n_H+rate[474]*x[46]*x[56]*n_H+rate[475]*x[51]*x[56]*n_H+rate[489]*x[34]*x[54]*n_H+rate[505]*x[43]+rate[522]*x[46]+rate[525]*x[51]+rate[526]*x[25]+rate[528]*x[34]+rate[579]*x[35]*x_e*n_H+rate[599]*x[43]*x[55]*zeta_H2+rate[608]*x[46]*x[55]*zeta_H2+rate[613]*x[51]*x[55]*zeta_H2+rate[625]*x[34]*x[55]*zeta_H2;
  ode[49] = form+x[49]*loss;
  loss = -rate[112]-rate[134]-rate[594]*zeta_He;
  form = +rate[66]*x[9]*x[55]*n_H+rate[74]*x[9]*x[56]*n_H+rate[76]*x[9]*x[49]*n_H+rate[77]*x[9]*x[36]*n_H+rate[78]*x[9]*x[46]*n_H+rate[79]*x[9]*x[42]*n_H+rate[80]*x[9]*x[18]*n_H+rate[81]*x[9]*x[21]*n_H+rate[296]*x[9]*x[55]*n_H+rate[355]*x[9]*x[38]*n_H+rate[356]*x[9]*x[38]*n_H+rate[357]*x[9]*x[44]*n_H+rate[358]*x[9]*x[36]*n_H+rate[359]*x[9]*x[36]*n_H+rate[360]*x[9]*x[36]*n_H+rate[361]*x[9]*x[36]*n_H+rate[362]*x[9]*x[46]*n_H+rate[363]*x[9]*x[51]*n_H+rate[364]*x[9]*x[34]*n_H+rate[365]*x[9]*x[34]*n_H+rate[366]*x[8]*x[9]*n_H+rate[367]*x[8]*x[9]*n_H+rate[368]*x[8]*x[9]*n_H+rate[369]*x[9]*x[42]*n_H+rate[370]*x[9]*x[42]*n_H+rate[371]*x[9]*x[18]*n_H+rate[372]*x[9]*x[18]*n_H+rate[373]*x[4]*x[9]*n_H+rate[374]*x[4]*x[9]*n_H+rate[375]*x[5]*x[9]*n_H+rate[376]*x[5]*x[9]*n_H+rate[377]*x[9]*x[47]*n_H+rate[378]*x[9]*x[21]*n_H+rate[379]*x[9]*x[52]*n_H+rate[380]*x[9]*x[12]*n_H+rate[381]*x[9]*x[20]*n_H+rate[382]*x[9]*x[20]*n_H+rate[420]*x[1]*x[9]*n_H+rate[584]*x[9]*x_e*n_H;
  ode[50] = form+x[50]*loss;
  loss = -rate[61]*x[14]*n_H-rate[89]*x[24]*n_H-rate[107]-rate[123]-rate[242]*x[19]*n_H-rate[253]*x[16]*n_H-rate[280]*x[14]*n_H-rate[300]*x[29]*n_H-rate[318]*x[2]*n_H-rate[363]*x[9]*n_H-rate[399]*x[22]*n_H-rate[475]*x[56]*n_H-rate[525]-rate[613]*x[55]*zeta_H2;
  form = +rate[20]*x[30]*x[49]*n_H+rate[24]*x[30]*x[38]*n_H+rate[32]*x[30]*x[36]*n_H+rate[33]*x[30]*x[46]*n_H+rate[38]*x[18]*x[30]*n_H+rate[39]*x[21]*x[30]*n_H+rate[40]*x[30]*x[53]*n_H+rate[71]*x[30]*x[42]*n_H+rate[72]*x[30]*x[56]*n_H+rate[96]*x[30]*x[54]*n_H+rate[101]*x[30]*x[52]*n_H+rate[184]*x[41]*x_e*n_H+rate[200]*x[21]*x[35]*n_H+rate[203]*x[20]*x[35]*n_H+rate[209]*x[41]*x[49]*n_H+rate[233]*x[38]*x[41]*n_H+rate[262]*x[41]*x[46]*n_H+rate[308]*x[41]*x[42]*n_H+rate[345]*x[34]*x[41]*n_H+rate[346]*x[8]*x[41]*n_H+rate[347]*x[18]*x[41]*n_H+rate[348]*x[5]*x[41]*n_H+rate[349]*x[41]*x[47]*n_H+rate[350]*x[12]*x[41]*n_H+rate[351]*x[41]*x[53]*n_H+rate[352]*x[20]*x[41]*n_H+rate[408]*x[41]*x[52]*n_H+rate[419]*x[1]*x[41]*n_H+rate[428]*x[21]*x[49]*n_H+rate[429]*x[49]*x[52]*n_H+rate[433]*x[20]*x[49]*n_H+rate[436]*x[21]*x[38]*n_H+rate[437]*x[38]*x[54]*n_H+rate[438]*x[38]*x[54]*n_H+rate[447]*x[44]*x[54]*n_H+rate[453]*x[21]*x[46]*n_H+rate[454]*x[21]*x[46]*n_H+rate[455]*x[21]*x[46]*n_H+rate[456]*x[21]*x[46]*n_H+rate[457]*x[46]*x[54]*n_H+rate[461]*x[20]*x[46]*n_H+rate[488]*x[34]*x[54]*n_H+rate[492]*x[4]*x[54]*n_H+rate[498]*x[34]*x[52]*n_H+rate[562]*x[49]*x[54]*n_H;
  ode[51] = form+x[51]*loss;
  loss = -rate[8]*x[55]*n_H-rate[14]*x[56]*n_H-rate[54]*x[40]*n_H-rate[65]*x[14]*n_H-rate[93]*x[24]*n_H-rate[101]*x[30]*n_H-rate[139]-rate[201]*x[35]*n_H-rate[220]*x[43]*n_H-rate[286]*x[14]*n_H-rate[329]*x[2]*n_H-rate[379]*x[9]*n_H-rate[386]*x[24]*n_H-rate[402]*x[22]*n_H-rate[405]*x[16]*n_H-rate[406]*x[30]*n_H-rate[407]*x[29]*n_H-rate[408]*x[41]*n_H-rate[409]*x[48]*n_H-rate[429]*x[49]*n_H-rate[430]*x[49]*n_H-rate[440]*x[38]*n_H-rate[441]*x[38]*n_H-rate[448]*x[44]*n_H-rate[449]*x[44]*n_H-rate[451]*x[36]*n_H-rate[469]*x[55]*n_H-rate[481]*x[56]*n_H-rate[495]*x[54]*n_H-rate[498]*x[34]*n_H-rate[499]*x[18]*n_H-2*rate[500]*x[52]*n_H-rate[501]*x[53]*n_H-rate[551]-rate[552]-rate[576]*x[56]*n_H-rate[611]*x[55]*zeta_H2-rate[628]*x[56]*zeta_H;
  form = +rate[6]*x[42]*x[55]*n_H+rate[12]*x[42]*x[56]*n_H+rate[28]*x[22]*x[38]*n_H+rate[37]*x[22]*x[46]*n_H+rate[97]*x[22]*x[42]*n_H+rate[98]*x[18]*x[22]*n_H+rate[99]*x[21]*x[22]*n_H+rate[100]*x[22]*x[53]*n_H+rate[127]*x[42]+rate[166]*x[29]*x_e*n_H+rate[177]*x[33]*x_e*n_H+rate[178]*x[33]*x_e*n_H+rate[206]*x[29]*x[49]*n_H+rate[217]*x[21]*x[43]*n_H+rate[226]*x[21]*x[45]*n_H+rate[231]*x[29]*x[38]*n_H+rate[260]*x[29]*x[46]*n_H+rate[300]*x[29]*x[51]*n_H+rate[301]*x[29]*x[42]*n_H+rate[302]*x[18]*x[29]*n_H+rate[304]*x[29]*x[53]*n_H+rate[306]*x[30]*x[42]*n_H+rate[370]*x[9]*x[42]*n_H+rate[383]*x[24]*x[36]*n_H+rate[384]*x[18]*x[24]*n_H+rate[389]*x[19]*x[54]*n_H+rate[393]*x[37]*x[54]*n_H+rate[396]*x[28]*x[54]*n_H+rate[439]*x[38]*x[54]*n_H+rate[445]*x[42]*x[44]*n_H+rate[455]*x[21]*x[46]*n_H+rate[456]*x[21]*x[46]*n_H+rate[458]*x[46]*x[54]*n_H+2*rate[467]*x[21]*x[55]*n_H+rate[468]*x[54]*x[55]*n_H+rate[475]*x[51]*x[56]*n_H+rate[476]*x[42]*x[56]*n_H+rate[480]*x[21]*x[56]*n_H+rate[484]*x[20]*x[56]*n_H+rate[487]*x[36]*x[54]*n_H+2*rate[490]*x[42]*x[54]*n_H+rate[491]*x[18]*x[54]*n_H+rate[493]*x[47]*x[54]*n_H+rate[533]*x[42]+rate[575]*x[54]*x[56]*n_H+rate[612]*x[42]*x[55]*zeta_H2+rate[630]*x[42]*x[56]*zeta_H;
  ode[52] = form+x[52]*loss;
  loss = -rate[23]*x[43]*n_H-rate[40]*x[30]*n_H-rate[56]*x[40]*n_H-rate[70]*x[29]*n_H-rate[95]*x[27]*n_H-rate[100]*x[22]*n_H-rate[102]*x[35]*n_H-rate[103]*x[35]*n_H-rate[104]*x[37]*n_H-rate[105]*x[28]*n_H-rate[142]-rate[221]*x[43]*n_H-rate[222]*x[43]*n_H-rate[228]*x[45]*n_H-rate[239]*x[39]*n_H-rate[257]*x[16]*n_H-rate[304]*x[29]*n_H-rate[305]*x[29]*n_H-rate[331]*x[2]*n_H-rate[351]*x[41]*n_H-rate[387]*x[27]*n_H-rate[403]*x[22]*n_H-rate[404]*x[22]*n_H-rate[412]*x[26]*n_H-rate[442]*x[38]*n_H-rate[443]*x[38]*n_H-rate[450]*x[44]*n_H-rate[452]*x[36]*n_H-rate[459]*x[46]*n_H-rate[460]*x[46]*n_H-rate[470]*x[55]*n_H-rate[486]*x[21]*n_H-rate[501]*x[52]*n_H-rate[502]*x[47]*n_H-rate[503]*x[20]*n_H-rate[556]-rate[560]*x[35]*n_H-rate[564]*x[49]*n_H-rate[578]*x_e*n_H-rate[590]-rate[597]*zeta_H-rate[624]*x[55]*zeta_H2;
  form = +rate[86]*x[6]*x[48]*n_H+rate[125]*x[34]+rate[129]*x[18]+rate[132]*x[5]+rate[133]*x[47]+rate[140]*x[1]+2*rate[141]*x[12]+rate[143]*x[20]+rate[160]*x[25]*x_e*n_H+rate[168]*x[37]*x_e*n_H+rate[182]*x[26]*x_e*n_H+rate[185]*x[32]*x_e*n_H+rate[187]*x[28]*x_e*n_H+rate[188]*x[17]*x_e*n_H+2*rate[193]*x[13]*x_e*n_H+rate[194]*x[23]*x_e*n_H+rate[204]*x[20]*x[35]*n_H+rate[263]*x[28]*x[46]*n_H+rate[309]*x[28]*x[42]*n_H+rate[353]*x[18]*x[28]*n_H+rate[365]*x[9]*x[34]*n_H+rate[368]*x[8]*x[9]*n_H+rate[380]*x[9]*x[12]*n_H+rate[382]*x[9]*x[20]*n_H+rate[391]*x[25]*x[54]*n_H+rate[395]*x[32]*x[54]*n_H+rate[413]*x[1]*x[35]*n_H+rate[414]*x[1]*x[39]*n_H+rate[415]*x[1]*x[40]*n_H+rate[416]*x[1]*x[37]*n_H+rate[417]*x[1]*x[2]*n_H+rate[418]*x[1]*x[33]*n_H+rate[419]*x[1]*x[41]*n_H+rate[420]*x[1]*x[9]*n_H+rate[421]*x[1]*x[31]*n_H+rate[422]*x[1]*x[24]*n_H+2*rate[423]*x[1]*x[48]*n_H+rate[424]*x[1]*x[23]*n_H+rate[427]*x[47]*x[49]*n_H+rate[431]*x[12]*x[49]*n_H+rate[433]*x[20]*x[49]*n_H+rate[479]*x[47]*x[56]*n_H+rate[482]*x[12]*x[56]*n_H+rate[484]*x[20]*x[56]*n_H+rate[485]*x[47]*x[47]*n_H+rate[488]*x[34]*x[54]*n_H+rate[493]*x[47]*x[54]*n_H+rate[496]*x[12]*x[54]*n_H+rate[497]*x[20]*x[54]*n_H+rate[528]*x[34]+rate[536]*x[18]+rate[542]*x[28]+rate[544]*x[5]+rate[545]*x[47]+rate[553]*x[1]+2*rate[555]*x[12]+rate[557]*x[20]+rate[587]*x[48]*x_e*n_H+rate[614]*x[47]*x[55]*zeta_H2+2*rate[615]*x[12]*x[55]*zeta_H2+rate[616]*x[20]*x[55]*zeta_H2+rate[618]*x[18]*x[55]*zeta_H2+rate[622]*x[5]*x[55]*zeta_H2+rate[625]*x[34]*x[55]*zeta_H2;
  ode[53] = form+x[53]*loss;
  loss = -rate[1]*x[46]*n_H-rate[3]*x[1]*n_H-rate[53]*x[40]*n_H-rate[96]*x[30]*n_H-rate[113]-rate[138]-rate[219]*x[43]*n_H-rate[227]*x[45]*n_H-rate[238]*x[39]*n_H-rate[285]*x[14]*n_H-rate[327]*x[2]*n_H-rate[328]*x[2]*n_H-rate[389]*x[19]*n_H-rate[390]*x[16]*n_H-rate[391]*x[25]*n_H-rate[392]*x[29]*n_H-rate[393]*x[37]*n_H-rate[394]*x[37]*n_H-rate[395]*x[32]*n_H-rate[396]*x[28]*n_H-rate[397]*x[28]*n_H-rate[398]*x[22]*n_H-rate[437]*x[38]*n_H-rate[438]*x[38]*n_H-rate[439]*x[38]*n_H-rate[447]*x[44]*n_H-rate[457]*x[46]*n_H-rate[458]*x[46]*n_H-rate[468]*x[55]*n_H-rate[487]*x[36]*n_H-rate[488]*x[34]*n_H-rate[489]*x[34]*n_H-rate[490]*x[42]*n_H-rate[491]*x[18]*n_H-rate[492]*x[4]*n_H-rate[493]*x[47]*n_H-rate[494]*x[47]*n_H-rate[495]*x[52]*n_H-rate[496]*x[12]*n_H-rate[497]*x[20]*n_H-rate[559]*x[35]*n_H-rate[562]*x[49]*n_H-rate[575]*x[56]*n_H-2*rate[577]*x[54]*n_H-rate[596]*zeta_H;
  form = +2*rate[7]*x[21]*x[55]*n_H+rate[8]*x[52]*x[55]*n_H+2*rate[13]*x[21]*x[56]*n_H+rate[14]*x[52]*x[56]*n_H+rate[26]*x[24]*x[38]*n_H+rate[35]*x[24]*x[46]*n_H+rate[75]*x[24]*x[56]*n_H+rate[88]*x[24]*x[36]*n_H+rate[89]*x[24]*x[51]*n_H+rate[90]*x[24]*x[42]*n_H+rate[91]*x[18]*x[24]*n_H+rate[92]*x[21]*x[24]*n_H+rate[93]*x[24]*x[52]*n_H+rate[123]*x[51]+2*rate[137]*x[21]+rate[139]*x[52]+rate[143]*x[20]+rate[159]*x[30]*x_e*n_H+rate[164]*x[29]*x_e*n_H+rate[165]*x[29]*x_e*n_H+rate[176]*x[33]*x_e*n_H+2*rate[191]*x[27]*x_e*n_H+rate[192]*x[22]*x_e*n_H+rate[194]*x[23]*x_e*n_H+rate[199]*x[21]*x[35]*n_H+rate[202]*x[20]*x[35]*n_H+rate[211]*x[27]*x[49]*n_H+rate[212]*x[22]*x[49]*n_H+rate[218]*x[21]*x[43]*n_H+rate[234]*x[22]*x[38]*n_H+rate[249]*x[22]*x[36]*n_H+rate[265]*x[27]*x[46]*n_H+rate[266]*x[22]*x[46]*n_H+rate[363]*x[9]*x[51]*n_H+rate[378]*x[9]*x[21]*n_H+rate[381]*x[9]*x[20]*n_H+rate[387]*x[27]*x[53]*n_H+rate[388]*x[21]*x[48]*n_H+rate[399]*x[22]*x[51]*n_H+rate[400]*x[22]*x[42]*n_H+rate[401]*x[18]*x[22]*n_H+rate[402]*x[22]*x[52]*n_H+rate[403]*x[22]*x[53]*n_H+rate[406]*x[30]*x[52]*n_H+rate[407]*x[29]*x[52]*n_H+rate[422]*x[1]*x[24]*n_H+rate[428]*x[21]*x[49]*n_H+rate[430]*x[49]*x[52]*n_H+rate[432]*x[20]*x[49]*n_H+rate[441]*x[38]*x[52]*n_H+rate[448]*x[44]*x[52]*n_H+rate[453]*x[21]*x[46]*n_H+rate[454]*x[21]*x[46]*n_H+rate[480]*x[21]*x[56]*n_H+rate[481]*x[52]*x[56]*n_H+rate[483]*x[20]*x[56]*n_H+rate[486]*x[21]*x[53]*n_H+rate[500]*x[52]*x[52]*n_H+rate[503]*x[20]*x[53]*n_H+rate[524]*x[30]+rate[525]*x[51]+rate[547]*x[27]+2*rate[549]*x[21]+rate[551]*x[52]+rate[557]*x[20]+rate[586]*x[24]*x_e*n_H+2*rate[609]*x[21]*x[55]*zeta_H2+rate[611]*x[52]*x[55]*zeta_H2+rate[613]*x[51]*x[55]*zeta_H2+rate[616]*x[20]*x[55]*zeta_H2+rate[628]*x[52]*x[56]*zeta_H+2*rate[629]*x[21]*x[56]*zeta_H;
  ode[54] = form+x[54]*loss;
  loss = -rate[4]*x[46]*n_H-2*rate[5]*x[55]*n_H-rate[6]*x[42]*n_H-rate[7]*x[21]*n_H-rate[8]*x[52]*n_H-rate[9]*x_e*n_H-rate[11]*x[56]*n_H-rate[66]*x[9]*n_H-rate[108]-rate[109]-rate[110]-rate[281]*x[14]*n_H-rate[287]*x[35]*n_H-rate[288]*x[43]*n_H-rate[289]*x[45]*n_H-rate[290]*x[19]*n_H-rate[291]*x[30]*n_H-rate[292]*x[25]*n_H-rate[293]*x[29]*n_H-rate[294]*x[37]*n_H-rate[295]*x[28]*n_H-rate[296]*x[9]*n_H-rate[297]*x[24]*n_H-rate[298]*x[22]*n_H-rate[299]*x[48]*n_H-rate[462]*x[49]*n_H-rate[463]*x[38]*n_H-rate[464]*x[44]*n_H-rate[465]*x[46]*n_H-rate[466]*x[47]*n_H-rate[467]*x[21]*n_H-rate[468]*x[54]*n_H-rate[469]*x[52]*n_H-rate[470]*x[53]*n_H-rate[566]*x[35]*n_H-rate[567]*x[49]*n_H-rate[568]*x[39]*n_H-rate[569]*x[46]*n_H-rate[570]*x[28]*n_H-rate[571]*x[17]*n_H-rate[572]*x[48]*n_H-rate[589]-rate[593]*zeta_H2;
  form = +rate[4]*x[46]*x[55]*n_H+rate[5]*x[55]*x[55]*n_H+rate[6]*x[42]*x[55]*n_H+rate[7]*x[21]*x[55]*n_H+rate[8]*x[52]*x[55]*n_H+rate[58]*x[14]*x[38]*n_H+rate[59]*x[14]*x[36]*n_H+rate[60]*x[14]*x[46]*n_H+rate[61]*x[14]*x[51]*n_H+rate[62]*x[14]*x[42]*n_H+rate[63]*x[14]*x[18]*n_H+rate[64]*x[14]*x[21]*n_H+rate[65]*x[14]*x[52]*n_H+rate[73]*x[14]*x[56]*n_H+rate[120]*x[44]+rate[121]*x[36]+rate[126]*x[8]+rate[129]*x[18]+rate[146]*x[45]*x_e*n_H+rate[150]*x[39]*x_e*n_H+rate[154]*x[16]*x_e*n_H+rate[155]*x[16]*x_e*n_H+2*rate[158]*x[16]*x_e*n_H+rate[164]*x[29]*x_e*n_H+rate[171]*x[2]*x_e*n_H+rate[173]*x[15]*x_e*n_H+rate[176]*x[33]*x_e*n_H+rate[177]*x[33]*x_e*n_H+rate[180]*x[26]*x_e*n_H+rate[182]*x[26]*x_e*n_H+rate[183]*x[0]*x_e*n_H+rate[208]*x[33]*x[49]*n_H+rate[214]*x[42]*x[43]*n_H+rate[216]*x[18]*x[43]*n_H+rate[220]*x[43]*x[52]*n_H+rate[225]*x[18]*x[45]*n_H+rate[236]*x[18]*x[39]*n_H+rate[237]*x[39]*x[47]*n_H+rate[238]*x[39]*x[54]*n_H+rate[239]*x[39]*x[53]*n_H+rate[248]*x[28]*x[36]*n_H+rate[252]*x[36]*x[48]*n_H+rate[269]*x[38]*x[40]*n_H+rate[270]*x[36]*x[40]*n_H+rate[271]*x[18]*x[40]*n_H+rate[272]*x[18]*x[40]*n_H+rate[273]*x[4]*x[40]*n_H+rate[274]*x[40]*x[47]*n_H+rate[277]*x[14]*x[36]*n_H+rate[283]*x[14]*x[18]*n_H+2*rate[284]*x[14]*x[18]*n_H+rate[313]*x[2]*x[49]*n_H+rate[314]*x[2]*x[38]*n_H+rate[315]*x[2]*x[44]*n_H+rate[316]*x[2]*x[36]*n_H+rate[317]*x[2]*x[46]*n_H+rate[318]*x[2]*x[51]*n_H+rate[319]*x[2]*x[34]*n_H+rate[320]*x[2]*x[8]*n_H+rate[321]*x[2]*x[42]*n_H+rate[322]*x[2]*x[18]*n_H+rate[323]*x[2]*x[4]*n_H+rate[324]*x[2]*x[5]*n_H+rate[325]*x[2]*x[47]*n_H+rate[326]*x[2]*x[6]*n_H+rate[328]*x[2]*x[54]*n_H+rate[329]*x[2]*x[52]*n_H+rate[330]*x[2]*x[12]*n_H+rate[331]*x[2]*x[53]*n_H+rate[332]*x[2]*x[20]*n_H+rate[337]*x[43]*x[56]*n_H+rate[338]*x[45]*x[56]*n_H+rate[339]*x[39]*x[56]*n_H+rate[340]*x[19]*x[56]*n_H+rate[341]*x[16]*x[56]*n_H+rate[342]*x[37]*x[56]*n_H+rate[343]*x[26]*x[56]*n_H+rate[344]*x[28]*x[56]*n_H+rate[354]*x[18]*x[28]*n_H+rate[355]*x[9]*x[38]*n_H+rate[357]*x[9]*x[44]*n_H+rate[358]*x[9]*x[36]*n_H+rate[359]*x[9]*x[36]*n_H+rate[366]*x[8]*x[9]*n_H+rate[372]*x[9]*x[18]*n_H+rate[392]*x[29]*x[54]*n_H+rate[394]*x[37]*x[54]*n_H+rate[411]*x[18]*x[48]*n_H+rate[417]*x[1]*x[2]*n_H+rate[437]*x[38]*x[54]*n_H+rate[442]*x[38]*x[53]*n_H+rate[447]*x[44]*x[54]*n_H+rate[471]*x[38]*x[56]*n_H+rate[472]*x[44]*x[56]*n_H+rate[473]*x[36]*x[56]*n_H+rate[474]*x[46]*x[56]*n_H+rate[476]*x[42]*x[56]*n_H+rate[477]*x[18]*x[56]*n_H+rate[478]*x[4]*x[56]*n_H+rate[479]*x[47]*x[56]*n_H+rate[481]*x[52]*x[56]*n_H+rate[506]*x[45]+rate[511]*x[39]+rate[515]*x[44]+rate[516]*x[19]+rate[518]*x[36]+rate[521]*x[36]+rate[530]*x[8]+rate[536]*x[18]+rate[538]*x[2]+rate[588]*x[56]*n_H+rate[605]*x[44]*x[55]*zeta_H2+rate[606]*x[36]*x[55]*zeta_H2+rate[618]*x[18]*x[55]*zeta_H2+rate[623]*x[8]*x[55]*zeta_H2+rate[631]*x[36]*x[56]*zeta_H;
  ode[55] = form+x[55]*loss;
  loss = -rate[2]*x[1]*n_H-rate[10]*x[46]*n_H-rate[11]*x[55]*n_H-rate[12]*x[42]*n_H-rate[13]*x[21]*n_H-rate[14]*x[52]*n_H-rate[72]*x[30]*n_H-rate[73]*x[14]*n_H-rate[74]*x[9]*n_H-rate[75]*x[24]*n_H-rate[111]-rate[130]-rate[337]*x[43]*n_H-rate[338]*x[45]*n_H-rate[339]*x[39]*n_H-rate[340]*x[19]*n_H-rate[341]*x[16]*n_H-rate[342]*x[37]*n_H-rate[343]*x[26]*n_H-rate[344]*x[28]*n_H-rate[471]*x[38]*n_H-rate[472]*x[44]*n_H-rate[473]*x[36]*n_H-rate[474]*x[46]*n_H-rate[475]*x[51]*n_H-rate[476]*x[42]*n_H-rate[477]*x[18]*n_H-rate[478]*x[4]*n_H-rate[479]*x[47]*n_H-rate[480]*x[21]*n_H-rate[481]*x[52]*n_H-rate[482]*x[12]*n_H-rate[483]*x[20]*n_H-rate[484]*x[20]*n_H-rate[565]*x[40]*n_H-rate[573]*x[35]*n_H-rate[574]*x[49]*n_H-rate[575]*x[54]*n_H-rate[576]*x[52]*n_H-2*rate[588]*n_H-rate[592]*zeta_H-rate[633]*x[3]*n_H;
  form = +rate[4]*x[46]*x[55]*n_H+2*rate[5]*x[55]*x[55]*n_H+rate[6]*x[42]*x[55]*n_H+rate[8]*x[52]*x[55]*n_H+2*rate[9]*x[55]*x_e*n_H+2*rate[10]*x[46]*x[56]*n_H+3*rate[11]*x[55]*x[56]*n_H+2*rate[12]*x[42]*x[56]*n_H+rate[13]*x[21]*x[56]*n_H+2*rate[14]*x[52]*x[56]*n_H+rate[41]*x[38]*x[40]*n_H+rate[42]*x[40]*x[44]*n_H+rate[43]*x[36]*x[40]*n_H+rate[44]*x[40]*x[46]*n_H+rate[45]*x[34]*x[40]*n_H+rate[46]*x[8]*x[40]*n_H+rate[47]*x[40]*x[42]*n_H+rate[48]*x[18]*x[40]*n_H+rate[49]*x[5]*x[40]*n_H+rate[50]*x[40]*x[47]*n_H+rate[51]*x[6]*x[40]*n_H+rate[52]*x[21]*x[40]*n_H+rate[53]*x[40]*x[54]*n_H+rate[54]*x[40]*x[52]*n_H+rate[55]*x[12]*x[40]*n_H+rate[56]*x[40]*x[53]*n_H+rate[57]*x[20]*x[40]*n_H+rate[108]*x[55]+2*rate[110]*x[55]+rate[115]*x[43]+rate[117]*x[38]+rate[118]*x[44]+rate[122]*x[46]+rate[127]*x[42]+rate[133]*x[47]+rate[139]*x[52]+rate[145]*x[43]*x_e*n_H+2*rate[147]*x[45]*x_e*n_H+rate[148]*x[45]*x_e*n_H+rate[149]*x[39]*x_e*n_H+2*rate[151]*x[39]*x_e*n_H+2*rate[152]*x[19]*x_e*n_H+rate[153]*x[19]*x_e*n_H+rate[154]*x[16]*x_e*n_H+2*rate[156]*x[16]*x_e*n_H+rate[157]*x[16]*x_e*n_H+2*rate[161]*x[14]*x_e*n_H+2*rate[162]*x[11]*x_e*n_H+rate[163]*x[11]*x_e*n_H+2*rate[165]*x[29]*x_e*n_H+rate[166]*x[29]*x_e*n_H+rate[167]*x[37]*x_e*n_H+2*rate[168]*x[37]*x_e*n_H+rate[169]*x[10]*x_e*n_H+rate[171]*x[2]*x_e*n_H+3*rate[172]*x[2]*x_e*n_H+rate[173]*x[15]*x_e*n_H+rate[174]*x[15]*x_e*n_H+rate[175]*x[33]*x_e*n_H+rate[176]*x[33]*x_e*n_H+2*rate[178]*x[33]*x_e*n_H+rate[179]*x[26]*x_e*n_H+2*rate[181]*x[26]*x_e*n_H+rate[182]*x[26]*x_e*n_H+rate[184]*x[41]*x_e*n_H+rate[186]*x[32]*x_e*n_H+rate[187]*x[28]*x_e*n_H+rate[189]*x[17]*x_e*n_H+rate[190]*x[7]*x_e*n_H+rate[192]*x[22]*x_e*n_H+rate[196]*x[35]*x[42]*n_H+rate[197]*x[18]*x[35]*n_H+rate[198]*x[35]*x[47]*n_H+rate[201]*x[35]*x[52]*n_H+rate[207]*x[37]*x[49]*n_H+rate[210]*x[28]*x[49]*n_H+rate[219]*x[43]*x[54]*n_H+rate[221]*x[43]*x[53]*n_H+rate[223]*x[18]*x[45]*n_H+rate[225]*x[18]*x[45]*n_H+rate[227]*x[45]*x[54]*n_H+rate[228]*x[45]*x[53]*n_H+rate[235]*x[38]*x[48]*n_H+rate[240]*x[44]*x[48]*n_H+rate[251]*x[36]*x[48]*n_H+rate[252]*x[36]*x[48]*n_H+rate[256]*x[6]*x[16]*n_H+rate[264]*x[24]*x[46]*n_H+rate[267]*x[46]*x[48]*n_H+rate[272]*x[18]*x[40]*n_H+rate[275]*x[14]*x[49]*n_H+rate[276]*x[14]*x[38]*n_H+rate[277]*x[14]*x[36]*n_H+rate[278]*x[14]*x[36]*n_H+rate[279]*x[14]*x[46]*n_H+rate[280]*x[14]*x[51]*n_H+rate[281]*x[14]*x[55]*n_H+rate[282]*x[14]*x[42]*n_H+rate[283]*x[14]*x[18]*n_H+rate[285]*x[14]*x[54]*n_H+rate[286]*x[14]*x[52]*n_H+rate[287]*x[35]*x[55]*n_H+rate[288]*x[43]*x[55]*n_H+rate[289]*x[45]*x[55]*n_H+rate[290]*x[19]*x[55]*n_H+rate[291]*x[30]*x[55]*n_H+rate[292]*x[25]*x[55]*n_H+rate[293]*x[29]*x[55]*n_H+rate[294]*x[37]*x[55]*n_H+rate[295]*x[28]*x[55]*n_H+rate[296]*x[9]*x[55]*n_H+rate[297]*x[24]*x[55]*n_H+rate[298]*x[22]*x[55]*n_H+rate[299]*x[48]*x[55]*n_H+rate[305]*x[29]*x[53]*n_H+rate[326]*x[2]*x[6]*n_H+rate[327]*x[2]*x[54]*n_H+rate[356]*x[9]*x[38]*n_H+rate[358]*x[9]*x[36]*n_H+rate[360]*x[9]*x[36]*n_H+rate[362]*x[9]*x[46]*n_H+rate[369]*x[9]*x[42]*n_H+rate[371]*x[9]*x[18]*n_H+rate[373]*x[4]*x[9]*n_H+rate[376]*x[5]*x[9]*n_H+rate[377]*x[9]*x[47]*n_H+rate[379]*x[9]*x[52]*n_H+rate[386]*x[24]*x[52]*n_H+rate[397]*x[28]*x[54]*n_H+rate[398]*x[22]*x[54]*n_H+rate[404]*x[22]*x[53]*n_H+rate[409]*x[48]*x[52]*n_H+rate[410]*x[18]*x[48]*n_H+rate[412]*x[26]*x[53]*n_H+rate[415]*x[1]*x[40]*n_H+rate[417]*x[1]*x[2]*n_H+rate[418]*x[1]*x[33]*n_H+rate[419]*x[1]*x[41]*n_H+rate[426]*x[47]*x[49]*n_H+rate[429]*x[49]*x[52]*n_H+2*rate[438]*x[38]*x[54]*n_H+rate[443]*x[38]*x[53]*n_H+rate[447]*x[44]*x[54]*n_H+rate[450]*x[44]*x[53]*n_H+rate[453]*x[21]*x[46]*n_H+rate[454]*x[21]*x[46]*n_H+rate[457]*x[46]*x[54]*n_H+rate[459]*x[46]*x[53]*n_H+rate[462]*x[49]*x[55]*n_H+rate[463]*x[38]*x[55]*n_H+rate[464]*x[44]*x[55]*n_H+rate[465]*x[46]*x[55]*n_H+rate[466]*x[47]*x[55]*n_H+rate[468]*x[54]*x[55]*n_H+rate[469]*x[52]*x[55]*n_H+rate[470]*x[53]*x[55]*n_H+rate[494]*x[47]*x[54]*n_H+rate[495]*x[52]*x[54]*n_H+rate[501]*x[52]*x[53]*n_H+rate[502]*x[47]*x[53]*n_H+rate[507]*x[45]+rate[510]*x[38]+rate[512]*x[39]+rate[513]*x[44]+rate[517]*x[19]+rate[519]*x[36]+rate[521]*x[36]+rate[522]*x[46]+rate[529]*x[14]+rate[531]*x[29]+rate[533]*x[42]+rate[535]*x[18]+rate[537]*x[2]+rate[539]*x[41]+rate[541]*x[28]+rate[545]*x[47]+rate[550]*x[22]+rate[551]*x[52]+rate[581]*x[40]*x_e*n_H+2*rate[589]*x[55]+rate[600]*x[43]*x[55]*zeta_H2+rate[602]*x[38]*x[55]*zeta_H2+rate[603]*x[44]*x[55]*zeta_H2+rate[607]*x[36]*x[55]*zeta_H2+rate[608]*x[46]*x[55]*zeta_H2+rate[611]*x[52]*x[55]*zeta_H2+rate[612]*x[42]*x[55]*zeta_H2+rate[614]*x[47]*x[55]*zeta_H2+rate[620]*x[18]*x[55]*zeta_H2+rate[628]*x[52]*x[56]*zeta_H+rate[630]*x[42]*x[56]*zeta_H;
  ode[56] = form+x[56]*loss;

  /* Store the electron abundance in the user data */
  data->x_e = x_e;

  return(0);
}
/*=======================================================================*/
